##############################################################################
#
# Copyright (c) 2008 Vanguardistas LLC.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
import os
import unittest
import doctest
from pprint import pformat
from van.testing.layer import zcml_layer, wsgi_intercept_layer
from zope.component import getUtility
from zope.interface import Interface

try:
    import zope.testbrowser
    have_testbrowser = True
except:
    have_testbrowser = False

MARKER = 'MARKER'
_HERE = os.path.dirname(__file__)

def simple_app(environ, start_response):
    """Simplest possible application object"""
    marker = getUtility(Interface, name=u"test")
    response = pformat(environ) + "\nMarker: %s" % marker
    start_response('200 OK', [('Content-type', 'text/plain'),
                              ('X-Powered-By', 'WSGI'),
                              ('Content-length', len(response)),
                              ])
    return [response]

class ZCMLLayer:
    zcml = os.path.join(_HERE, 'ftesting.zcml')
zcml_layer(ZCMLLayer)
    
class ZCMLExtraLayer:
    zcml_features = ('extra',)
    zcml = os.path.join(_HERE, 'ftesting.zcml')
zcml_layer(ZCMLExtraLayer)

class FunctionalLayer(ZCMLLayer):
    @classmethod
    def make_application(cls):
        return simple_app
wsgi_intercept_layer(FunctionalLayer)

def test_suite():
    ftests = unittest.TestSuite([doctest.DocFileSuite('README.txt')])
    ftests.layer = FunctionalLayer
    extra_tests = unittest.TestSuite([doctest.DocFileSuite('zcml_features.txt')])
    extra_tests.layer = ZCMLExtraLayer
    if have_testbrowser:
        ftests.addTest(doctest.DocFileSuite('testbrowser.txt'))
    return unittest.TestSuite([
            ftests,
            extra_tests,
            doctest.DocTestSuite('van.testing.layer'),
            ])



##############################################################################
#
# Copyright (c) 2008 Vanguardistas LLC.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
from zope.configuration import config, xmlconfig
from zope.testing.cleanup import cleanUp
import wsgi_intercept

def null_layer(layer):
    """Sets up a class as a layer that does nothing.

    Useful if you want a layer to inherit from other layers but not inherit
    their methods. Inheriting the method can cause it to be triggered twice as
    the zope testrunner (zope.testing) walks the super-classes of the layer
    firing the setup methods.

        >>> class LayerA:
        ...     @classmethod
        ...     def setUp(cls):
        ...         print 'fired'
        ...     @classmethod
        ...     def tearDown(cls):
        ...         print 'fired'
        ...     @classmethod
        ...     def testTearDown(cls):
        ...         print 'fired'
        ...     @classmethod
        ...     def testSetUp(cls):
        ...         print 'fired'

    If we just subclass the method, then our subclass will inherit the methods:

        >>> class LayerB(LayerA):
        ...     pass
        >>> LayerB.setUp()
        fired
        >>> LayerB.tearDown()
        fired
        >>> LayerB.testSetUp()
        fired
        >>> LayerB.testTearDown()
        fired

    So, we override

        >>> null_layer(LayerB)
        >>> LayerB.setUp()
        >>> LayerB.tearDown()
        >>> LayerB.testSetUp()
        >>> LayerB.testTearDown()
    """
    def null(cls):
        pass
    layer.setUp = classmethod(null)
    layer.tearDown = layer.setUp
    layer.testSetUp = layer.setUp
    layer.testTearDown = layer.setUp


def zcml_layer(layer):
    """Sets up a class as a ZCMLLayer.

    The class can have 3 attributes:
        zcml: The site zcml file to load.
        zcml_features: A tuple of strings indicating the features to load.
        allow_teardown: Whether to allow teardown of this layer.
                        Default: True
    """

    def setUp(cls):
        features = getattr(cls, 'zcml_features', ())
        context = config.ConfigurationMachine()
        xmlconfig.registerCommonDirectives(context)
        for feature in features:
            context.provideFeature(feature)
        context = xmlconfig.file(cls.zcml, context=context)
    layer.setUp = classmethod(setUp)

    def tearDown(cls):
        cleanUp()
        if not getattr(cls, 'allow_teardown', True):
            raise NotImplementedError
    layer.tearDown = classmethod(tearDown)

    def null(cls):
        pass
    layer.testSetUp = classmethod(null)
    layer.testTearDown = classmethod(null)


class ErrorHandler:
    """This middleware sets up the environ so that handleErrors actually works.

    zope.testbrowser.Browser.handleErrors that is.
    """
    def __init__(self, application):
        self.application = application

    def __call__(self, environ, start_response):
        if environ.get('HTTP_X_ZOPE_HANDLE_ERRORS') == 'False':
            environ['wsgi.handleErrors'] = False
        if 'HTTP_X_ZOPE_HANDLE_ERRORS' in environ:
            del environ['HTTP_X_ZOPE_HANDLE_ERRORS']
        def my_start_response(status, headers, exc_info=None):
            # Behave like zope.testbrowser.testing:
            # sort response headers and insert status code in first place.
            headers = sorted(headers)
            headers.insert(0, ('Status', status))
            return start_response(status, headers, exc_info=exc_info)
        return self.application(environ, my_start_response)


def wsgi_intercept_layer(layer):

    domain = getattr(layer, 'domain', 'localhost')
    port = getattr(layer, 'port', 80)

    def make_debug_application():
        return ErrorHandler(layer.make_application())

    def setUp(cls):
        wsgi_intercept.add_wsgi_intercept(domain, port, make_debug_application)
    layer.setUp = classmethod(setUp)

    def tearDown(cls):
        wsgi_intercept.remove_wsgi_intercept(domain, port)

    def null(cls):
        pass
    layer.testSetUp = classmethod(null)
    layer.testTearDown = classmethod(null)



##############################################################################
#
# Copyright (c) 2008 Vanguardistas LLC.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

import os
from setuptools import setup, find_packages

long_description = (
    '.. contents::\n\n'
    + open(os.path.join('van', 'testing', 'README.txt')).read()
    + '\n\n'
    + open(os.path.join('CHANGES.txt')).read()
    )

setup(name="van.testing",
      version='3.0.0',
      license='ZPL 2.1',
      url='http://pypi.python.org/pypi/van.testing',
      author_email='zope-dev@zope.org',
      author="Vanguardistas LLC",
      description="Layers for zope.testing to simplify test setups",
      packages=find_packages(),
      namespace_packages=["van"],
      install_requires=[
          'setuptools',
          'wsgi_intercept',
          'zope.component',
          'zope.configuration',
          'zope.testing',
          ],
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Zope Public License',
        'Programming Language :: Python',
        'Topic :: Software Development :: Testing',
        'Development Status :: 5 - Production/Stable',
        ],
      long_description=long_description,
      include_package_data = True,
      zip_safe = False,
      )
